package com.example.a3;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.StrictMode;
import android.util.Log;
import android.widget.Toast;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Locale;

public class AsyncStockNameLoader extends AsyncTask<String, Integer, String>{

    @SuppressLint("StaticFieldLeak")
    private MainActivity mainActivity;
    private static  String DATA_URL = "https://api.iextrading.com/1.0/ref-data/symbols";
    private static final String TAG = "AsyncStockNameLoader";
    private String requestedSymbol;


    AsyncStockNameLoader(MainActivity ma) {
        mainActivity = ma;
    }

    //Post execute method created after main activity:
    public void onPostExecute(String s){

        ArrayList<String> allNames = parseJSON(s);
        //Empty list:
        if (allNames == null){
            return;
        }
        if (allNames.size() > 1){
            multiStock(allNames.toArray(new String[allNames.size()])).show();
        }
        else {
            //We do a singular version to add in the stock:
            String symbolNew = allNames.get(0);
            String stockSymbol = symbolNew.split(":")[0];
            String companySymbol = symbolNew.split(":")[1];
            mainActivity.getNumberData(stockSymbol,companySymbol);
        }

    }


    ///////////////////////////DIALOGS FOR ERRORS//////////////////////////////////////////////////
    public Dialog invalidStock(){
        AlertDialog.Builder builder = new AlertDialog.Builder((mainActivity));
        builder.setMessage("Symbol Not Found: " + requestedSymbol);
        return builder.create();
    }

    //Function that will display list of stocks to choose from for the user:
    public Dialog multiStock (final String[] stocks){
        AlertDialog.Builder builder = new AlertDialog.Builder((mainActivity));
        builder.setTitle("Please Make Selection: ");
        builder.setItems(stocks, new DialogInterface.OnClickListener() {
            @Override
            //Pass information over to MainActivity:
            public void onClick(DialogInterface dialog, int pos) {
                String symbolNew = stocks[pos];
                String stockSymbol = symbolNew.split(":")[0];
                String companySymbol = symbolNew.split(":")[0];
                mainActivity.getNumberData(stockSymbol,companySymbol);
            }
        });
        //Other option that is NEVERMIND, displayed on the list:
        builder.setNegativeButton("NEVERMIND", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                return;
            }
        });
        return builder.create();
    }


    @Override
    protected String doInBackground(String... params) {
        String dataUrl = DATA_URL;
        Uri dataUri = Uri.parse(dataUrl);
        String urlToUse = dataUri.toString();
        StringBuilder sb = new StringBuilder();

        try {
            URL url = new URL(urlToUse);

            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            Log.d(TAG, "doInBackground: " + params);
            Log.d(TAG, "doInBackground: ResponseCode: " + conn.getResponseCode());

            conn.setRequestMethod("GET");

            InputStream is = conn.getInputStream();
            BufferedReader reader = new BufferedReader((new InputStreamReader(is)));
            //Log.d(TAG, "doInBackground: " + sb);
            String line;
            while ((line = reader.readLine()) != null) {
                sb.append(line).append('\n');
            }

            Log.d(TAG, "doInBackground: " + sb.toString());


        } catch (Exception e) {
            Log.e(TAG, "doInBackground: ", e);
            return null;
        }

        return sb.toString();
    }

    //Function that will parse our item and add it into a list of stocks:
    //I was originally using this as another item of stocks list, however I found errors in main
    //due to it. Therefore, for the names, it is just an array list of strings, for the full info
    //it is an array list of stocks
    private ArrayList<String> parseJSON(String s) {

        ArrayList<String> stockNameList = new ArrayList<>();

        try {
            JSONArray jObjMain = new JSONArray(s);
            for (int i = 0; i < jObjMain.length(); i++) {
                JSONObject jStock = (JSONObject) jObjMain.get(i);
                String symbol = jStock.getString("symbol");
                String companyName = jStock.getString("name");
                stockNameList.add(symbol + ":" + companyName);
            }
            return stockNameList;
        } catch (Exception e) {
            Log.d(TAG, "parseJSON: " + e.getMessage());
            e.printStackTrace();
        }
        return null;
    }

}

