package com.example.a3;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import android.app.Dialog;
import android.content.ContentValues;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.text.InputFilter;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.widget.Toast;

import android.content.Context;
import android.content.DialogInterface;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.text.InputType;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.EditText;

import java.lang.reflect.Array;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/*
John William Lynch
CSC 472 : Assignment 3
Stock Watcher App
https://github.com/siliu/StockWatch/blob/master/app/src/main/java/com/example/siliu/stockwatch/MainActivity.java

 */
public class MainActivity extends AppCompatActivity implements View.OnClickListener, View.OnLongClickListener {

    //Create our objects to be used with recylcer view and swipe refresh layout:
    private RecyclerView recyclerView;

    //Swipe refresh layout
    private SwipeRefreshLayout swiper;
    //Private list for all stock objects:
    private List<Stock> userInputStockList = new ArrayList<>();

    //Create adapter for our stocks as well:
    private StockAdapter stockAdapter;
    private boolean firstStart;
    private static final String TAG = "MainActivity";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        //Set title to Stock Watch:
        setTitle("Stock Watch");

        //Recycler view creation:
        recyclerView = findViewById(R.id.mainRecyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        stockAdapter = new StockAdapter(userInputStockList, this);
        recyclerView.setAdapter(stockAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));


        for (int i = 0; i < 5; i++) {
            userInputStockList.add(new Stock("A", "Test", 0, 0, 0));
        }


        swiper = (SwipeRefreshLayout) findViewById(R.id.swiper);
        swiper.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                doRefresh();
            }
        });

    }

    /*
    Two functions below create the menu and operate on them:
     */
    //Need to create and show the options menu on our main activity:
    public boolean onCreateOptionsMenu(Menu menu){
        getMenuInflater().inflate(R.menu.action_menu, menu);
        return true;
    }

    //When the PLUS button is selected, need to prompt the user to enter in
    //CAPITAL Letters to select their respective stock:
    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch (item.getItemId()){
            case R.id.addItemIcon:
                //We want to ensure that if the internet is not connected, the user cannot
                //use the plus functionality:
                if (doNetCheck() == false){
                    //Toast.makeText(this, "Not connected, can't use plus", Toast.LENGTH_SHORT).show();
                    disonnectedAlert();
                }
                else if (doNetCheck() == true){
                    //Toast.makeText(this, "Connected, can use:", Toast.LENGTH_SHORT).show();
                    addNewStockItem();
                }
        }
        return super.onOptionsItemSelected(item);
    }

    /*
    Method to be used alongside the AsyncStockNumberLoader that will add a full stock to our collection:
     */
    public void addFullStockItem(Stock newStock) {
        //Stock is empty:
        if (newStock == null) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle("Symbol Not Found: " + newStock.getSymbol());
            builder.setMessage("No data for this stock symbol.");
            builder.show();
            return;
            }

        //Stock is a duplicate:
        if (userInputStockList.contains(newStock)){
            duplicateAlert(newStock).show();
            return;
        }
        //Else it is neither, and then we can add it:
        userInputStockList.add(newStock);

        Collections.sort(userInputStockList, new Comparator<Stock>() {
            @Override
            public int compare(Stock o1, Stock o2) {
                return o1.getSymbol().compareTo(o2.getSymbol());
            }
        });
        stockAdapter.notifyDataSetChanged();
    }

    /*
    Method that loads up all items upon startup:
     */
    public void loadAllFullStocks (Stock newStocks){
        //Small function that adds all elements of current stock to aux one:
        userInputStockList.add(newStocks);
        Collections.sort(userInputStockList, new Comparator<Stock>() {
            @Override
            public int compare(Stock o1, Stock o2) {
                return o1.getSymbol().compareTo(o2.getSymbol());
            }
        });
        stockAdapter.notifyDataSetChanged();
    }



    //Function that operates to use the swipe refresh method:
    //Need to target the created SWIPER that will take in information:
    private void doRefresh() {
        //A swipe will reset/reload data, we need to double check if still connected during this as well:
        //Keep copy of current stocks:
        boolean firstStart = true;

        if (doNetCheck() == true){
            List<Stock> newStocks = new ArrayList<>(userInputStockList);
            userInputStockList.clear();
            for (int i = 0; i < newStocks.size(); i++){
                String newSymbol = newStocks.get(i).getSymbol();
                String newCompanyName = newStocks.get(i).getCompanyName();
                new AsyncStockNumberLoader(MainActivity.this, firstStart).execute(newSymbol, newCompanyName);

            }
        }
        swiper.setRefreshing(false);
    }


    //////////////////////////////ON-CLICK METHODS FOR STOCKS//////////////////////////////////////
    @Override
    public void onClick(View v) {
        int pos = recyclerView.getChildLayoutPosition(v);
        Stock clickedStock = userInputStockList.get(pos);
        String url = "https://www.marketwatch.com/investing/stock/" + clickedStock.getSymbol();
        Intent intent = new Intent(Intent.ACTION_VIEW);
        intent.setData(Uri.parse(url));
        startActivity(intent);

    }

    @Override
    public boolean onLongClick(View v) {
        deleteAlert(v).show();
        return false;
    }

    //Info received from AsyncStockNameLoader: now we will pass it over again to AsyncNumberLoader
    //so that we can access the other URL using the symbol and API key:
    public void getNumberData (String symbol, String companyName){
        firstStart = false;
        new AsyncStockNumberLoader(MainActivity.this, firstStart).execute(symbol, companyName);
    }

    /*
    //Function for updating data inside of the  class: this loads in
    //every single symbol in the large json file:
    public void updateData(ArrayList<String[]> sList) {
        for (int a = 0; a < sList.size(); a++){
            String symbol = sList.get(a)[0];
            String companyName = sList.get(a)[1];
            new AsyncStockNumberLoader(this).execute(symbol, companyName);
        }
        stockAdapter.notifyDataSetChanged();
    }
    */


    //////////////////////////////////////DIALOGS///////////////////////////////////////////////////
    /*
    Below Are all DIALOG Activities that can occur in three seperate Tasks: Main, AsyncName, AsyncNum:
     */

    //Small aux functions that just check the connection of the app:
        /*
    Functions that operate on checking network connection.
    Aux function that is used for the action menu to return if the internet is connected:
     */

    private boolean doNetCheck() {
        ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        if (cm == null) {
            Toast.makeText(this, "Cannot access ConnectivityManager", Toast.LENGTH_SHORT).show();
            return false;
        }

        NetworkInfo netInfo = cm.getActiveNetworkInfo();
        //If it is connected, return true. If not connected, return false:
        if (netInfo != null && netInfo.isConnected()) {
            return true;
        } else {
            return false;
        }
    }

    //Dialog that runs when the user is not connected to the Internet:
        /*
    Method that runs if the user attempts to press plus while they are not connected to the internet:
     */
    private void disonnectedAlert () {
        if (doNetCheck() == false) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle("No Network Connection");
            builder.setMessage("Stocks Cannot Be Added Without A Network Connection");
            builder.show();
        }
    }

    //Dialog that runs when the user attempts to REFRESH their list while they are not connected:
    private void disconnectedAlertRefresh () {
        if (doNetCheck() == false) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle("No Network Connection");
            builder.setMessage("Stocks Cannot Be Added Without A Network Connection");
            builder.show();
        }
    }


    private Dialog deleteAlert(final View v){
        int pos = recyclerView.getChildLayoutPosition(v);
        final Stock currentStock = userInputStockList.get(pos);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);


        builder.setTitle("Delete Stock");
        builder.setMessage("Delete Stock Symbol " + currentStock.getSymbol());
        builder.setPositiveButton("DELETE", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                if (!(userInputStockList.isEmpty())){
                    String currentStockSymbol = currentStock.getSymbol();
                    userInputStockList.remove(currentStock);
                    stockAdapter.notifyDataSetChanged();
                    Toast.makeText(v.getContext(), "Stock " + currentStockSymbol + " deleted...", Toast.LENGTH_SHORT).show();
                }

            }
        });

        builder.setNegativeButton("CANCEL", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                Toast.makeText(v.getContext(), "Deletion Cancelled...", Toast.LENGTH_SHORT).show();
            }
        });
        builder.setIcon(R.drawable.ic_audiotrack_dark);
        return builder.create();
    }

    //Dialog that runs if we have a duplicated stock:
    private Dialog duplicateAlert (Stock symbol) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle("Duplicate Stock");
            builder.setMessage("Stock Symbol " + symbol + " is already displayed");
            //builder.setIcon(R.drawable.ic_launcher);
            AlertDialog dialog = builder.create();
            return dialog;
    }

    /*
    Function that runs once the user presses the plus button:
     */
    public void addNewStockItem(){
        //Another secure check to ensure that we are connected to the internet:
        if (doNetCheck() == true){
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle("Stock Selection");
            builder.setMessage("Please enter a Stock Symbol:");
            final EditText requestedSymbol = new EditText(this);
            requestedSymbol.setInputType(InputType.TYPE_CLASS_TEXT | Gravity.CENTER);
            requestedSymbol.setInputType(Gravity.CENTER);
            requestedSymbol.setFilters(new InputFilter[] {new InputFilter.AllCaps()});
            builder.setView(requestedSymbol);
            builder.setPositiveButton("Ok", new DialogInterface.OnClickListener() {

                public void onClick(DialogInterface dialog, int id) {
                    if (requestedSymbol.getText().toString().isEmpty()) {
                        Toast.makeText(MainActivity.this, "Cannot Leave Selection Blank:", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    //If not blank, two scenarios: it is a new stock, or it is a duplicated stock:
                    boolean isInputted = false;
                    for (int i= 0; i < userInputStockList.size(); i++){
                        String currentSymbol = userInputStockList.get(i).getSymbol();
                        if (currentSymbol.equals(requestedSymbol)) {
                            isInputted = true;
                        }
                        //We have a duplicate:
                        if (isInputted) {
                            duplicateAlert(userInputStockList.get(i));
                        }
                        //Else we do not have a duplicate:
                        else {
                            String [] allSymbols = {
                                    currentSymbol
                            };
                            new AsyncStockNameLoader(MainActivity.this).execute(allSymbols);
                        }

                    }
                }
            });
            //Else the user cancelled, and close dialog
            builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    dialog.cancel();
                }
            });
            builder.show();
        }
    }
    //////////////////////////////////////DIALOGS///////////////////////////////////////////////////
}
